

---------------------------------------------
--             MINIMAP.LUA                 --
-- draws a real-size minimap on your radar --
---------------------------------------------
--               BANAAN                    --
--       http://servers.cs2d.nl            --
--          servers@cs2d.nl                --
--              24-12-11                   --
---------------------------------------------






-----------------------------------
--     DEFINES (or SETTINGS)     --
-----------------------------------

local ENABLE_CACHE = true      -- should parsed maps be saved for future use?
local RADIUS   = 12            -- radar consists of 25 x 25 tiles, with 13|13 being your own position
local CACHEDIR = 'maps/mcache' -- folder where map caches should be stored, will be created by the script
local SHAPE = 'circle'         -- or 'square'


-- If this runs on windows, the path seperator should be '\' instead of '/'
if os.getenv('WinDir') then CACHEDIR:gsub('/', '\\') end


-- Skeleton
MiniMap = {
	map = {},
	img = {},
	
	draw   = function(id) MiniMap:Draw(id) end,
	hide   = function(id) for _, v in pairs(MiniMap.img[id]) do imagealpha(v, 0) end end,
	reset  = function(id) MiniMap.img = {} end,
	delete = function(id) for _, v in pairs(MiniMap.img[id]) do freeimage(v) end MiniMap.img[id] = {} end
}


-----------------------------------
--   HOOKS (a bit inefficient)   --
-----------------------------------
addhook("movetile", "MiniMap.draw")
addhook("die", "MiniMap.hide")
addhook("startround", "MiniMap.reset")
addhook("leave", "MiniMap.delete")






-----------------------------------
--   FUNCTIONS (boring stuff)    --
-----------------------------------

--      MiniMap:New()      --
-- Load map properties, either from cache or create new properties

function MiniMap:New()
	if ENABLE_CACHE then
		local fp = io.open(CACHEDIR .. "/" .. map("name") .. ".mc", "r")
		if fp then
			loadstring("MiniMap.map = " .. fp:read("*all")) ()
			fp:close()
		
		else
			MiniMap:ParseMap()
			local fp = io.open(CACHEDIR .. "/" .. map("name") .. ".mc", "w")
			if fp then
				fp:write(self:T2S(self.map))
				fp:close()
			end
		end
	
	else MiniMap:ParseMap() end
end





--      MiniMap:ParseMap()     --
-- Iterate over the map to create a grid (two-dimentional array) of tile properties
-- This will be stored in the table MiniMap.map
-- Properties: 0 = floor,
--             1 = obstacle,
--             2 = wall,
--             3 = deadly,
--             4 = water

function MiniMap:ParseMap()
	local ys = map("ysize")
	for x = 0, map("xsize") do
		self.map[x] = {}
		for y = 0, ys do
			if     tile(x, y, "obstacle")       then self.map[x][y] = 1
			elseif tile(x, y, "wall")           then self.map[x][y] = 2
			elseif tile(x, y, "deadly")         then self.map[x][y] = 3
			elseif tile(x, y, "property") == 14 then self.map[x][y] = 4
			else                                     self.map[x][y] = 0
			end
		end
	end
end





--      MiniMap:T2S(table)      --
-- Convert a table to a string, which can later be converted back using loadstring()
-- This specific implementation is only meant for minimaps
-- Returns the converted string

function MiniMap:T2S(t)
	local s = "{"
	for i, v in pairs(t) do
		if i == 0 then s = s .. "[0]=" end
		if type(v) == "number" then s = s .. tostring(v) .. ","
		elseif type(v) == "table" then s = s .. self:T2S(v) .. "," end
	end
	if s:sub(-1) == "," then s = s:sub(1, -2) end
	s = s .. "}"
	return s
end





--      MiniMap:Draw(id)      --
-- Draw the minimap and create images if necessary

function MiniMap:Draw(id)
	if not self.img[id] then self.img[id] = {} end
	
	local xp, yp = player(id, "tilex"), player(id, "tiley")
	local mx, my = xp - RADIUS, yp - RADIUS
	local yl, im = 1, 1
	
	for x = 1, RADIUS * 2 + 1 do
		for y = 1, RADIUS * 2 + 1 do
			if SHAPE == 'square' or math.sqrt((x - RADIUS - 1)^2 + (y - RADIUS - 1)^2) <= RADIUS then
				local p, n
				if self.map[mx + (x - 1)] == nil then p, n = 0, 0
				else p, n = self.map[mx + (x - 1)][my + (y - 1)] or 0, self.map[mx + (x - 1)][my + y] or 0 end
				
				if p ~= 0 then
					if p == n and ((shape == 'square' and y < RADIUS * 2 + 1) or math.sqrt((x - RADIUS - 1)^2 + (y - RADIUS)^2) <= RADIUS) then
						yl = yl + 1

					else
						if not self.img[id][im] then self.img[id][im] = image("gfx/minimap_p.bmp", 0, 1, 2, id) end
						imagepos(self.img[id][im], 1 + x * 4, 1 + (y - (yl / 2)) * 4, 0)
						imagescale(self.img[id][im], 4, 4 * yl)
						
						if p == 1 then -- obstacle
							imagealpha(self.img[id][im], 0.3)
							imagecolor(self.img[id][im], 255, 255, 255)
						elseif p == 2 then -- wall
							imagealpha(self.img[id][im], 1)
							imagecolor(self.img[id][im], 175, 255, 175)
						elseif p == 3 then -- deadly
							imagealpha(self.img[id][im], 1)
							imagecolor(self.img[id][im], 255, 0, 0)
						elseif p == 4 then -- water
							imagealpha(self.img[id][im], 0.3)
							imagecolor(self.img[id][im], 0, 0, 255)
						end
						
						yl, im = 1, im + 1
					end
				end
			end
		end
	end

	if #self.img[id] > im then
		for i = im, #self.img[id] do imagealpha(self.img[id][i], 0) end
	end
end





--      MiniMap:MkDir()      --
-- If CACHEDIR does not exist, attempt to create it
-- If creating fails, disable caching

function MiniMap:MkDir()
	if not os.rename(CACHEDIR, CACHEDIR) then
		if os.execute("mkdir " .. CACHEDIR) ~= 0 then ENABLE_CACHE = false end
	end
end
MiniMap:MkDir()




-- Load map grid
MiniMap:New()




-- Banaan -- 241211 --
